@@ -35,7 +35,7 @@ $ sudo mn --controller=remote --topo linear,2
 """  # noqa
 
 from socket import error as SocketError
-from ryu.contrib.tinyrpc.exc import InvalidReplyError
+from tinyrpc.exc import InvalidReplyError
 
 
 from ryu.app.wsgi import (
@@ -83,6 +83,11 @@ class WebSocketTopology(app_manager.RyuApp):
         msg = ev.link.to_dict()
         self._rpc_broadcall('event_link_delete', msg)
 
+    @set_ev_cls(event.EventHostAdd)
+    def _event_host_add_handler(self, ev):
+        msg = ev.host.to_dict()
+        self._rpc_broadcall('event_host_add', msg)
+
     def _rpc_broadcall(self, func_name, msg):
         disconnected_clients = []
         for rpc_client in self.rpc_clients:
