@@ -44,6 +44,23 @@ class EventDPBase(event.EventBase):
 
 
 class EventDP(EventDPBase):
+    """
+    An event class to notify connect/disconnect of a switch.
+
+    For OpenFlow switches, one can get the same notification by observing
+    ryu.controller.ofp_event.EventOFPStateChange.
+    An instance has at least the following attributes.
+
+    ========= =================================================================
+    Attribute Description
+    ========= =================================================================
+    dp        A ryu.controller.controller.Datapath instance of the switch
+    enter     True when the switch connected to our controller.  False for
+              disconnect.
+    ports     A list of port instances.
+    ========= =================================================================
+    """
+
     def __init__(self, dp, enter_leave):
         # enter_leave
         # True: dp entered
@@ -53,6 +70,13 @@ class EventDP(EventDPBase):
         self.ports = []  # port list when enter or leave
 
 
+class EventDPReconnected(EventDPBase):
+    def __init__(self, dp):
+        super(EventDPReconnected, self).__init__(dp)
+        # port list, which should not change across reconnects
+        self.ports = []
+
+
 class EventPortBase(EventDPBase):
     def __init__(self, dp, port):
         super(EventPortBase, self).__init__(dp)
@@ -60,16 +84,64 @@ class EventPortBase(EventDPBase):
 
 
 class EventPortAdd(EventPortBase):
+    """
+    An event class for switch port status "ADD" notification.
+
+    This event is generated when a new port is added to a switch.
+    For OpenFlow switches, one can get the same notification by observing
+    ryu.controller.ofp_event.EventOFPPortStatus.
+    An instance has at least the following attributes.
+
+    ========= =================================================================
+    Attribute Description
+    ========= =================================================================
+    dp        A ryu.controller.controller.Datapath instance of the switch
+    port      port number
+    ========= =================================================================
+    """
+
     def __init__(self, dp, port):
         super(EventPortAdd, self).__init__(dp, port)
 
 
 class EventPortDelete(EventPortBase):
+    """
+    An event class for switch port status "DELETE" notification.
+
+    This event is generated when a port is removed from a switch.
+    For OpenFlow switches, one can get the same notification by observing
+    ryu.controller.ofp_event.EventOFPPortStatus.
+    An instance has at least the following attributes.
+
+    ========= =================================================================
+    Attribute Description
+    ========= =================================================================
+    dp        A ryu.controller.controller.Datapath instance of the switch
+    port      port number
+    ========= =================================================================
+    """
+
     def __init__(self, dp, port):
         super(EventPortDelete, self).__init__(dp, port)
 
 
 class EventPortModify(EventPortBase):
+    """
+    An event class for switch port status "MODIFY" notification.
+
+    This event is generated when some attribute of a port is changed.
+    For OpenFlow switches, one can get the same notification by observing
+    ryu.controller.ofp_event.EventOFPPortStatus.
+    An instance has at least the following attributes.
+
+    ========= ====================================================================
+    Attribute Description
+    ========= ====================================================================
+    dp        A ryu.controller.controller.Datapath instance of the switch
+    port      port number
+    ========= ====================================================================
+    """
+
     def __init__(self, dp, new_port):
         super(EventPortModify, self).__init__(dp, new_port)
 
@@ -113,11 +185,14 @@ class DPSet(app_manager.RyuApp):
         # - forget the older connection as it likely will disappear soon
         # - do not send EventDP leave/enter events
         # - keep the PortState for the dpid
+        send_dp_reconnected = False
         if dp.id in self.dps:
             self.logger.warning('DPSET: Multiple connections from %s',
                                 dpid_to_str(dp.id))
             self.logger.debug('DPSET: Forgetting datapath %s', self.dps[dp.id])
+            (self.dps[dp.id]).close()
             self.logger.debug('DPSET: New datapath %s', dp)
+            send_dp_reconnected = True
         self.dps[dp.id] = dp
         if dp.id not in self.port_state:
             self.port_state[dp.id] = PortState()
@@ -128,6 +203,10 @@ class DPSet(app_manager.RyuApp):
                     self._port_added(dp, port)
                     ev.ports.append(port)
             self.send_event_to_observers(ev)
+        if send_dp_reconnected:
+            ev = EventDPReconnected(dp)
+            ev.ports = self.port_state.get(dp.id, {}).values()
+            self.send_event_to_observers(ev)
 
     def _unregister(self, dp):
         # see the comment in _register().
@@ -139,7 +218,7 @@ class DPSet(app_manager.RyuApp):
         # Now datapath is already dead, so port status change event doesn't
         # interfere us.
         ev = EventDP(dp, False)
-        for port in self.port_state.get(dp.id, {}).values():
+        for port in list(self.port_state.get(dp.id, {}).values()):
             self._port_deleted(dp, port)
             ev.ports.append(port)
 
@@ -152,7 +231,6 @@ class DPSet(app_manager.RyuApp):
         """
         This method returns the ryu.controller.controller.Datapath
         instance for the given Datapath ID.
-        Raises KeyError if no such a datapath connected to this controller.
         """
         return self.dps.get(dp_id)
 
@@ -176,7 +254,7 @@ class DPSet(app_manager.RyuApp):
 
     @set_ev_cls(ofp_event.EventOFPStateChange,
                 [handler.MAIN_DISPATCHER, handler.DEAD_DISPATCHER])
-    def dispacher_change(self, ev):
+    def dispatcher_change(self, ev):
         datapath = ev.datapath
         assert datapath is not None
         if ev.state == handler.MAIN_DISPATCHER:
