@@ -16,19 +16,32 @@
 
 
 class EventBase(object):
-    # Nothing yet
-    pass
+    """
+    The base of all event classes.
+
+    A Ryu application can define its own event type by creating a subclass.
+    """
+
+    def __init__(self):
+        super(EventBase, self).__init__()
 
 
 class EventRequestBase(EventBase):
+    """
+    The base class for synchronous request for RyuApp.send_request.
+    """
     def __init__(self):
         super(EventRequestBase, self).__init__()
         self.dst = None  # app.name of provide the event.
         self.src = None
         self.sync = False
+        self.reply_q = None
 
 
 class EventReplyBase(EventBase):
+    """
+    The base class for synchronous request reply for RyuApp.send_reply.
+    """
     def __init__(self, dst):
         super(EventReplyBase, self).__init__()
         self.dst = dst
