@@ -18,23 +18,55 @@ import collections
 
 from ryu.base import app_manager
 import ryu.exception as ryu_exc
-from ryu.app.rest_nw_id import NW_ID_UNKNOWN
 from ryu.controller import event
 from ryu.exception import NetworkNotFound, NetworkAlreadyExist
 from ryu.exception import PortAlreadyExist, PortNotFound, PortUnknown
 
 
+NW_ID_UNKNOWN = '__NW_ID_UNKNOWN__'
+
+
 class MacAddressAlreadyExist(ryu_exc.RyuException):
     message = 'port (%(dpid)s, %(port)s) has already mac %(mac_address)s'
 
 
 class EventNetworkDel(event.EventBase):
+    """
+    An event class for network deletion.
+
+    This event is generated when a network is deleted by the REST API.
+    An instance has at least the following attributes.
+
+    ========== ===================================================================
+    Attribute  Description
+    ========== ===================================================================
+    network_id Network ID
+    ========== ===================================================================
+    """
+
     def __init__(self, network_id):
         super(EventNetworkDel, self).__init__()
         self.network_id = network_id
 
 
 class EventNetworkPort(event.EventBase):
+    """
+    An event class for notification of port arrival and deperture.
+
+    This event is generated when a port is introduced to or removed from a
+    network by the REST API.
+    An instance has at least the following attributes.
+
+    ========== ================================================================
+    Attribute  Description
+    ========== ================================================================
+    network_id Network ID
+    dpid       OpenFlow Datapath ID of the switch to which the port belongs.
+    port_no    OpenFlow port number of the port
+    add_del    True for adding a port.  False for removing a port.
+    ========== ================================================================
+    """
+
     def __init__(self, network_id, dpid, port_no, add_del):
         super(EventNetworkPort, self).__init__()
         self.network_id = network_id
@@ -44,6 +76,26 @@ class EventNetworkPort(event.EventBase):
 
 
 class EventMacAddress(event.EventBase):
+    """
+    An event class for end-point MAC address registration.
+
+    This event is generated when a end-point MAC address is updated
+    by the REST API.
+    An instance has at least the following attributes.
+
+    =========== ===============================================================
+    Attribute   Description
+    =========== ===============================================================
+    network_id  Network ID
+    dpid        OpenFlow Datapath ID of the switch to which the port belongs.
+    port_no     OpenFlow port number of the port
+    mac_address The old MAC address of the port if add_del is False.  Otherwise
+                the new MAC address.
+    add_del     False if this event is a result of a port removal.  Otherwise
+                True.
+    =========== ===============================================================
+    """
+
     def __init__(self, dpid, port_no, network_id, mac_address, add_del):
         super(EventMacAddress, self).__init__()
         assert network_id is not None
@@ -62,7 +114,7 @@ class Networks(dict):
         self.send_event = f
 
     def list_networks(self):
-        return self.keys()
+        return list(self.keys())
 
     def has_network(self, network_id):
         return network_id in self
@@ -183,7 +235,7 @@ class DPIDs(dict):
 
     def get_ports(self, dpid, network_id=None, mac_address=None):
         if network_id is None:
-            return self.get(dpid, {}).values()
+            return list(self.get(dpid, {}).values())
         if mac_address is None:
             return [p for p in self.get(dpid, {}).values()
                     if p.network_id == network_id]
