@@ -17,6 +17,8 @@
 import logging
 import os
 
+import netaddr
+
 
 # We don't bother to use cfg.py because monkey patch needs to be
 # called very early. Instead, we use an environment variable to
@@ -27,6 +29,10 @@ LOG = logging.getLogger('ryu.lib.hub')
 
 if HUB_TYPE == 'eventlet':
     import eventlet
+    # HACK:
+    # sleep() is the workaround for the following issue.
+    # https://github.com/eventlet/eventlet/issues/401
+    eventlet.sleep()
     import eventlet.event
     import eventlet.queue
     import eventlet.semaphore
@@ -110,11 +116,16 @@ if HUB_TYPE == 'eventlet':
             assert backlog is None
             assert spawn == 'default'
 
-            if ':' in listen_info[0]:
+            if netaddr.valid_ipv6(listen_info[0]):
                 self.server = eventlet.listen(listen_info,
                                               family=socket.AF_INET6)
+            elif os.path.isdir(os.path.dirname(listen_info[0])):
+                # Case for Unix domain socket
+                self.server = eventlet.listen(listen_info[0],
+                                              family=socket.AF_UNIX)
             else:
                 self.server = eventlet.listen(listen_info)
+
             if ssl_args:
                 def wrap_and_handle(sock, addr):
                     ssl_args.setdefault('server_side', True)
