@@ -16,8 +16,7 @@
 """
 BFD Control packet parser/serializer
 
-RFC 5880
-BFD Control packet format
+[RFC 5880] BFD Control packet format::
 
     0                   1                   2                   3
     0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
@@ -35,47 +34,46 @@ BFD Control packet format
    |                 Required Min Echo RX Interval                 |
    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 
-   An optional Authentication Section MAY be present in the following
-   format of types:
+An optional Authentication Section MAY be present in the following
+format of types:
 
-   1. Format of Simple Password Authentication Section
+1. Format of Simple Password Authentication Section::
 
-    0                   1                   2                   3
-    0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
-   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
-   |   Auth Type   |   Auth Len    |  Auth Key ID  |  Password...  |
-   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
-   |                              ...                              |
-   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+        0                   1                   2                   3
+        0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
+       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+       |   Auth Type   |   Auth Len    |  Auth Key ID  |  Password...  |
+       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+       |                              ...                              |
+       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 
-   2. Format of Keyed MD5 and Meticulous Keyed MD5 Authentication Section
+2. Format of Keyed MD5 and Meticulous Keyed MD5 Authentication Section::
 
-    0                   1                   2                   3
-    0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
-   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
-   |   Auth Type   |   Auth Len    |  Auth Key ID  |   Reserved    |
-   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
-   |                        Sequence Number                        |
-   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
-   |                      Auth Key/Digest...                       |
-   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
-   |                              ...                              |
-   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+        0                   1                   2                   3
+        0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
+       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+       |   Auth Type   |   Auth Len    |  Auth Key ID  |   Reserved    |
+       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+       |                        Sequence Number                        |
+       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+       |                      Auth Key/Digest...                       |
+       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+       |                              ...                              |
+       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 
-   3. Format of Keyed SHA1 and Meticulous Keyed SHA1 Authentication Section
-
-    0                   1                   2                   3
-    0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
-   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
-   |   Auth Type   |   Auth Len    |  Auth Key ID  |   Reserved    |
-   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
-   |                        Sequence Number                        |
-   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
-   |                       Auth Key/Hash...                        |
-   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
-   |                              ...                              |
-   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+3. Format of Keyed SHA1 and Meticulous Keyed SHA1 Authentication Section::
 
+        0                   1                   2                   3
+        0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
+       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+       |   Auth Type   |   Auth Len    |  Auth Key ID  |   Reserved    |
+       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+       |                        Sequence Number                        |
+       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+       |                       Auth Key/Hash...                        |
+       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+       |                              ...                              |
+       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 """
 import binascii
 import hashlib
@@ -417,7 +415,7 @@ class SimplePassword(BFDAuth):
         return self.serialize_hdr() + \
             struct.pack(self._PACK_STR, self.auth_key_id) + self.password
 
-    def authenticate(self, prev=None, auth_keys={}):
+    def authenticate(self, prev=None, auth_keys=None):
         """Authenticate the password for this packet.
 
         This method can be invoked only when ``self.password`` is defined.
@@ -431,6 +429,7 @@ class SimplePassword(BFDAuth):
         ``auth_keys`` is a dictionary of authentication key chain which
         key is an integer of *Auth Key ID* and value is a string of *Password*.
         """
+        auth_keys = auth_keys if auth_keys else {}
         assert isinstance(prev, bfd)
         if self.auth_key_id in auth_keys and \
                 self.password == auth_keys[self.auth_key_id]:
@@ -520,7 +519,7 @@ class KeyedMD5(BFDAuth):
         return auth_hdr_bin + struct.pack(self._PACK_STR, self.auth_key_id, 0,
                                           self.seq, self.digest)
 
-    def authenticate(self, prev, auth_keys={}):
+    def authenticate(self, prev, auth_keys=None):
         """Authenticate the MD5 digest for this packet.
 
         This method can be invoked only when ``self.digest`` is defined.
@@ -536,6 +535,7 @@ class KeyedMD5(BFDAuth):
         ``auth_keys`` is a dictionary of authentication key chain which
         key is an integer of *Auth Key ID* and value is a string of *Auth Key*.
         """
+        auth_keys = auth_keys if auth_keys else {}
         assert isinstance(prev, bfd)
 
         if self.digest is None:
@@ -670,7 +670,7 @@ class KeyedSHA1(BFDAuth):
         return auth_hdr_bin + struct.pack(self._PACK_STR, self.auth_key_id, 0,
                                           self.seq, self.auth_hash)
 
-    def authenticate(self, prev, auth_keys={}):
+    def authenticate(self, prev, auth_keys=None):
         """Authenticate the SHA1 hash for this packet.
 
         This method can be invoked only when ``self.auth_hash`` is defined.
@@ -686,6 +686,7 @@ class KeyedSHA1(BFDAuth):
         ``auth_keys`` is a dictionary of authentication key chain which
         key is an integer of *Auth Key ID* and value is a string of *Auth Key*.
         """
+        auth_keys = auth_keys if auth_keys else {}
         assert isinstance(prev, bfd)
 
         if self.auth_hash is None:
