@@ -17,8 +17,9 @@ import struct
 from . import packet_base
 from . import vlan
 from . import mpls
-from ryu.ofproto import ether
+from . import ether_types as ether
 from ryu.lib import addrconv
+from ryu.lib.pack_utils import msg_pack_into
 
 
 class ethernet(packet_base.PacketBase):
@@ -39,8 +40,15 @@ class ethernet(packet_base.PacketBase):
 
     _PACK_STR = '!6s6sH'
     _MIN_LEN = struct.calcsize(_PACK_STR)
+    _MIN_PAYLOAD_LEN = 46
+    _TYPE = {
+        'ascii': [
+            'src', 'dst'
+        ]
+    }
 
-    def __init__(self, dst, src, ethertype):
+    def __init__(self, dst='ff:ff:ff:ff:ff:ff', src='00:00:00:00:00:00',
+                 ethertype=ether.ETH_TYPE_IP):
         super(ethernet, self).__init__()
         self.dst = dst
         self.src = src
@@ -55,6 +63,11 @@ class ethernet(packet_base.PacketBase):
                 buf[ethernet._MIN_LEN:])
 
     def serialize(self, payload, prev):
+        # Append padding if the payload is less than 46 bytes long
+        pad_len = self._MIN_PAYLOAD_LEN - len(payload)
+        if pad_len > 0:
+            payload.extend(b'\x00' * pad_len)
+
         return struct.pack(ethernet._PACK_STR,
                            addrconv.mac.text_to_bin(self.dst),
                            addrconv.mac.text_to_bin(self.src),
