@@ -16,8 +16,7 @@
 """
 Internet Group Management Protocol(IGMP) packet parser/serializer
 
-RFC 1112
-IGMP v1 format
+[RFC 1112] IGMP v1 format::
 
     0                   1                   2                   3
     0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
@@ -27,8 +26,7 @@ IGMP v1 format
    |                         Group Address                         |
    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 
-RFC 2236
-IGMP v2 format
+[RFC 2236] IGMP v2 format::
 
     0                   1                   2                   3
     0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
@@ -38,8 +36,7 @@ IGMP v2 format
    |                         Group Address                         |
    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 
-RFC 3376
-IGMP v3 Membership Query format
+[RFC 3376] IGMP v3 Membership Query format::
 
     0                   1                   2                   3
     0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
@@ -60,7 +57,7 @@ IGMP v3 Membership Query format
    |                       Source Address [N]                      |
    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 
-IGMP v3 Membership Report format
+IGMP v3 Membership Report format::
 
     0                   1                   2                   3
     0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
@@ -92,7 +89,7 @@ IGMP v3 Membership Report format
    |                                                               |
    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 
-where each Group Record has the following internal format:
+Where each Group Record has the following internal format::
 
    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
    |  Record Type  |  Aux Data Len |     Number of Sources (N)     |
@@ -119,6 +116,7 @@ where each Group Record has the following internal format:
 
 import six
 import struct
+from math import trunc
 
 from ryu.lib import addrconv
 from ryu.lib import stringify
@@ -209,7 +207,7 @@ class igmp(packet_base.PacketBase):
 
     def serialize(self, payload, prev):
         hdr = bytearray(struct.pack(self._PACK_STR, self.msgtype,
-                        self.maxresp, self.csum,
+                        trunc(self.maxresp), self.csum,
                         addrconv.ipv4.text_to_bin(self.address)))
 
         if self.csum == 0:
@@ -299,9 +297,9 @@ class igmpv3_query(igmp):
     def serialize(self, payload, prev):
         s_qrv = self.s_flg << 3 | self.qrv
         buf = bytearray(struct.pack(self._PACK_STR, self.msgtype,
-                        self.maxresp, self.csum,
+                        trunc(self.maxresp), self.csum,
                         addrconv.ipv4.text_to_bin(self.address),
-                        s_qrv, self.qqic, self.num))
+                        s_qrv, trunc(self.qqic), self.num))
         for src in self.srcs:
             buf.extend(struct.pack('4s', addrconv.ipv4.text_to_bin(src)))
         if 0 == self.num:
