@@ -15,12 +15,31 @@
 
 import inspect
 import struct
+import base64
+
+import six
 
 from . import packet_base
 from . import ethernet
 
+from ryu import utils
+from ryu.lib.stringify import StringifyMixin
 
-class Packet(object):
+
+# Packet class dictionary
+mod = inspect.getmembers(utils.import_module("ryu.lib.packet"),
+                         lambda cls: (inspect.ismodule(cls)))
+cls_list = []
+for _, m in mod:
+    cl = inspect.getmembers(m,
+                            lambda cls: (
+                                inspect.isclass(cls) and
+                                issubclass(cls, packet_base.PacketBase)))
+    cls_list.extend(list(cl))
+PKT_CLS_DICT = dict(cls_list)
+
+
+class Packet(StringifyMixin):
     """A packet decoder/encoder class.
 
     An instance is used to either decode or encode a single packet.
@@ -34,6 +53,9 @@ class Packet(object):
     *data* should be omitted when encoding a packet.
     """
 
+    # Ignore data field when outputting json representation.
+    _base_attributes = ['data']
+
     def __init__(self, data=None, protocols=None, parse_cls=ethernet.ethernet):
         super(Packet, self).__init__()
         self.data = data
@@ -47,13 +69,17 @@ class Packet(object):
     def _parser(self, cls):
         rest_data = self.data
         while cls:
+            # Ignores an empty buffer
+            if not six.binary_type(rest_data).strip(b'\x00'):
+                break
             try:
                 proto, cls, rest_data = cls.parser(rest_data)
             except struct.error:
                 break
             if proto:
                 self.protocols.append(proto)
-        if rest_data:
+        # If rest_data is all padding, we ignore rest_data
+        if rest_data and six.binary_type(rest_data).strip(b'\x00'):
             self.protocols.append(rest_data)
 
     def serialize(self):
@@ -72,8 +98,22 @@ class Packet(object):
                     prev = r[i + 1]
                 data = p.serialize(self.data, prev)
             else:
-                data = str(p)
-            self.data = data + self.data
+                data = six.binary_type(p)
+            self.data = bytearray(data + self.data)
+
+    @classmethod
+    def from_jsondict(cls, dict_, decode_string=base64.b64decode,
+                      **additional_args):
+        protocols = []
+        for proto in dict_['protocols']:
+            for key, value in proto.items():
+                if key in PKT_CLS_DICT:
+                    pkt_cls = PKT_CLS_DICT[key]
+                    protocols.append(pkt_cls.from_jsondict(value))
+                else:
+                    raise ValueError('unknown protocol name %s' % key)
+
+        return cls(protocols=protocols)
 
     def add_protocol(self, proto):
         """Register a protocol *proto* for this packet.
@@ -109,6 +149,9 @@ class Packet(object):
         self.add_protocol(trailer)
         return self
 
+    def __truediv__(self, trailer):
+        return self.__div__(trailer)
+
     def __iter__(self):
         return iter(self.protocols)
 
@@ -143,3 +186,4 @@ def _PacketBase__div__(self, trailer):
     return pkt
 
 packet_base.PacketBase.__div__ = _PacketBase__div__
+packet_base.PacketBase.__truediv__ = _PacketBase__div__
