@@ -17,8 +17,8 @@
 """
 VRRP packet parser/serializer
 
-RFC 3768
-VRRP v2 packet format
+[RFC 3768] VRRP v2 packet format::
+
     0                   1                   2                   3
     0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
@@ -40,8 +40,8 @@ VRRP v2 packet format
    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 
 
-RFC 5798
-VRRP v3 packet format
+[RFC 5798] VRRP v3 packet format::
+
      0                   1                   2                   3
      0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
@@ -64,20 +64,19 @@ VRRP v3 packet format
     +                                                               +
     |                                                               |
     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
-
 """
 
 import struct
 
 from ryu.lib.packet import ethernet
+from ryu.lib.packet import ether_types as ether
+from ryu.lib.packet import in_proto as inet
 from ryu.lib.packet import ipv4
 from ryu.lib.packet import ipv6
 from ryu.lib.packet import packet
 from ryu.lib.packet import packet_base
 from ryu.lib.packet import packet_utils
 from ryu.lib.packet import vlan
-from ryu.ofproto import ether
-from ryu.ofproto import inet
 from ryu.lib import addrconv
 
 
@@ -275,12 +274,12 @@ class vrrp(packet_base.PacketBase):
 
         self.checksum = checksum
         self.ip_addresses = ip_addresses
-        assert len(ip_addresses) == self.count_ip
+        assert len(list(ip_addresses)) == self.count_ip
 
         self.auth_type = auth_type
         self.auth_data = auth_data
 
-        self._is_ipv6 = is_ipv6(self.ip_addresses[0])
+        self._is_ipv6 = is_ipv6(list(self.ip_addresses)[0])
         self.identification = 0         # used for ipv4 identification
 
     def checksum_ok(self, ipvx, vrrp_buf):
@@ -355,7 +354,7 @@ class vrrp(packet_base.PacketBase):
                            inet.IPPROTO_VRRP, VRRP_IPV6_HOP_LIMIT,
                            primary_ip_address, VRRP_IPV6_DST_ADDRESS)
         else:
-            header_length = ipv4.ipv4._MIN_LEN / 4      # XXX _MIN_LEN
+            header_length = ipv4.ipv4._MIN_LEN // 4      # XXX _MIN_LEN
             total_length = 0
             tos = 0xc0  # set tos to internetwork control
             identification = self.get_identification()
@@ -463,8 +462,7 @@ class vrrpv2(vrrp):
         ip_addresses_pack_str = cls._ip_addresses_pack_str(count_ip)
         ip_addresses_bin = struct.unpack_from(ip_addresses_pack_str, buf,
                                               offset)
-        ip_addresses = map(lambda x: addrconv.ipv4.bin_to_text(x),
-                           ip_addresses_bin)
+        ip_addresses = [addrconv.ipv4.bin_to_text(x) for x in ip_addresses_bin]
 
         offset += struct.calcsize(ip_addresses_pack_str)
         auth_data = struct.unpack_from(cls._AUTH_DATA_PACK_STR, buf, offset)
@@ -499,8 +497,7 @@ class vrrpv2(vrrp):
                          vrrp_.checksum)
         offset += vrrpv2._MIN_LEN
         struct.pack_into(ip_addresses_pack_str, buf, offset,
-                         *map(lambda x: addrconv.ipv4.text_to_bin(x),
-                              vrrp_.ip_addresses))
+                         *[addrconv.ipv4.text_to_bin(x) for x in vrrp_.ip_addresses])
         offset += ip_addresses_len
         struct.pack_into(vrrpv2._AUTH_DATA_PACK_STR, buf, offset,
                          *vrrp_.auth_data)
@@ -574,7 +571,7 @@ class vrrpv3(vrrp):
         max_adver_int &= VRRP_V3_MAX_ADVER_INT_MASK
 
         offset = cls._MIN_LEN
-        address_len = (len(buf) - offset) / count_ip
+        address_len = (len(buf) - offset) // count_ip
         # Address version (IPv4 or IPv6) is determined by network layer
         # header type.
         # Unfortunately it isn't available. Guess it by vrrp packet length.
@@ -590,7 +587,7 @@ class vrrpv3(vrrp):
                     address_len, count_ip))
 
         ip_addresses_bin = struct.unpack_from(pack_str, buf, offset)
-        ip_addresses = map(lambda x: conv(x), ip_addresses_bin)
+        ip_addresses = [conv(x) for x in ip_addresses_bin]
         msg = cls(version, type_, vrid, priority,
                   count_ip, max_adver_int, checksum, ip_addresses)
         return msg, None, buf[len(msg):]
@@ -624,7 +621,7 @@ class vrrpv3(vrrp):
                          vrrp_.vrid, vrrp_.priority,
                          vrrp_.count_ip, vrrp_.max_adver_int, vrrp_.checksum)
         struct.pack_into(ip_addresses_pack_str, buf, vrrpv3._MIN_LEN,
-                         *map(lambda x: conv(x), vrrp_.ip_addresses))
+                         *[conv(x) for x in vrrp_.ip_addresses])
 
         if checksum:
             vrrp_.checksum = packet_utils.checksum_ip(prev, len(buf), buf)
