@@ -1,5 +1,3 @@
-#!/usr/bin/env python
-#
 # Copyright (C) 2013 Nippon Telegraph and Telephone Corporation.
 # Copyright (C) 2013 YAMAMOTO Takashi <yamamoto at valinux co jp>
 #
@@ -16,10 +14,17 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-# msgpack-rpc
-# http://wiki.msgpack.org/display/MSGPACK/RPC+specification
+# Specification:
+# - msgpack
+#   https://github.com/msgpack/msgpack/blob/master/spec.md
+# - msgpack-rpc
+#   https://github.com/msgpack-rpc/msgpack-rpc/blob/master/spec.md
+
+from collections import deque
+import select
 
 import msgpack
+import six
 
 
 class MessageType(object):
@@ -34,13 +39,8 @@ class MessageEncoder(object):
     """
     def __init__(self):
         super(MessageEncoder, self).__init__()
-        # note: on-wire msgpack has no notion of encoding.
-        # the msgpack-python library implicitly converts unicode to
-        # utf-8 encoded bytes by default.  we don't want to rely on
-        # the behaviour though because it seems to be going to change.
-        # cf. https://gist.github.com/methane/5022403
-        self._packer = msgpack.Packer(encoding=None)
-        self._unpacker = msgpack.Unpacker(encoding=None)
+        self._packer = msgpack.Packer(encoding='utf-8', use_bin_type=True)
+        self._unpacker = msgpack.Unpacker(encoding='utf-8')
         self._next_msgid = 0
 
     def _create_msgid(self):
@@ -49,20 +49,20 @@ class MessageEncoder(object):
         return this_id
 
     def create_request(self, method, params):
-        assert isinstance(method, str)
+        assert isinstance(method, (str, six.binary_type))
         assert isinstance(params, list)
         msgid = self._create_msgid()
-        return (self._packer.pack([MessageType.REQUEST, msgid, method,
-                                  params]), msgid)
+        return (self._packer.pack(
+            [MessageType.REQUEST, msgid, method, params]), msgid)
 
     def create_response(self, msgid, error=None, result=None):
         assert isinstance(msgid, int)
-        assert 0 <= msgid and msgid <= 0xffffffff
+        assert 0 <= msgid <= 0xffffffff
         assert error is None or result is None
         return self._packer.pack([MessageType.RESPONSE, msgid, error, result])
 
     def create_notification(self, method, params):
-        assert isinstance(method, str)
+        assert isinstance(method, (str, six.binary_type))
         assert isinstance(params, list)
         return self._packer.pack([MessageType.NOTIFY, method, params])
 
@@ -75,21 +75,18 @@ class MessageEncoder(object):
         for m in self._unpacker:
             self._dispatch_message(m, disp_table)
 
-    def _dispatch_message(self, m, disp_table):
+    @staticmethod
+    def _dispatch_message(m, disp_table):
         # XXX validation
-        type = m[0]
+        t = m[0]
         try:
-            f = disp_table[type]
+            f = disp_table[t]
         except KeyError:
             # ignore messages with unknown type
             return
         f(m[1:])
 
 
-from collections import deque
-import select
-
-
 class EndPoint(object):
     """An endpoint
     *sock* is a socket-like.  it can be either blocking or non-blocking.
@@ -230,7 +227,7 @@ class EndPoint(object):
         except KeyError:
             return None
         error, result = m
-        return (result, error)
+        return result, error
 
     def get_notification(self):
         return self._get_message(self._notifications)
@@ -240,6 +237,7 @@ class RPCError(Exception):
     """an error from server
     """
     def __init__(self, error):
+        super(RPCError, self).__init__()
         self._error = error
 
     def get_value(self):
