@@ -1,4 +1,4 @@
-# Copyright (C) 2011, 2012 Nippon Telegraph and Telephone Corporation.
+# Copyright (C) 2011-2015 Nippon Telegraph and Telephone Corporation.
 # Copyright (C) 2011, 2012 Isaku Yamahata <yamahata at valinux co jp>
 # Copyright (C) 2012 Simon Horman <horms ad verge net au>
 #
@@ -20,9 +20,11 @@ import struct
 from ryu import exception
 from ryu.lib import mac
 from ryu.lib.pack_utils import msg_pack_into
-from . import ofproto_parser
-from . import ofproto_v1_0
-from . import inet
+from ryu.ofproto import ether
+from ryu.ofproto import ofproto_parser
+from ryu.ofproto import ofproto_v1_0
+from ryu.ofproto import inet
+
 
 import logging
 LOG = logging.getLogger('ryu.ofproto.nx_match')
@@ -91,6 +93,7 @@ class Flow(ofproto_parser.StringifyMixin):
         self.regs = [0] * FLOW_N_REGS
         self.ipv6_label = 0
         self.pkt_mark = 0
+        self.tcp_flags = 0
 
 
 class FlowWildcards(ofproto_parser.StringifyMixin):
@@ -113,6 +116,7 @@ class FlowWildcards(ofproto_parser.StringifyMixin):
         self.regs_mask = [0] * FLOW_N_REGS
         self.wildcards = ofproto_v1_0.OFPFW_ALL
         self.pkt_mark_mask = 0
+        self.tcp_flags_mask = 0
 
 
 class ClsRule(ofproto_parser.StringifyMixin):
@@ -309,6 +313,10 @@ class ClsRule(ofproto_parser.StringifyMixin):
         self.flow.pkt_mark = pkt_mark
         self.wc.pkt_mark_mask = mask
 
+    def set_tcp_flags(self, tcp_flags, mask):
+        self.flow.tcp_flags = tcp_flags
+        self.wc.tcp_flags_mask = mask
+
     def flow_format(self):
         # Tunnel ID is only supported by NXM
         if self.wc.tun_id_mask != 0:
@@ -329,6 +337,9 @@ class ClsRule(ofproto_parser.StringifyMixin):
         if self.wc.regs_bits > 0:
             return ofproto_v1_0.NXFF_NXM
 
+        if self.flow.tcp_flags > 0:
+            return ofproto_v1_0.NXFF_NXM
+
         return ofproto_v1_0.NXFF_OPENFLOW10
 
     def match_tuple(self):
@@ -945,6 +956,19 @@ class MFPktMark(MFField):
                          rule.wc.pkt_mark_mask)
 
 
+@_register_make
+@_set_nxm_headers([ofproto_v1_0.NXM_NX_TCP_FLAGS,
+                   ofproto_v1_0.NXM_NX_TCP_FLAGS_W])
+class MFTcpFlags(MFField):
+    @classmethod
+    def make(cls, header):
+        return cls(header, MF_PACK_STRING_BE16)
+
+    def put(self, buf, offset, rule):
+        return self.putm(buf, offset, rule.flow.tcp_flags,
+                         rule.wc.tcp_flags_mask)
+
+
 def serialize_nxm_match(rule, buf, offset):
     old_offset = offset
 
@@ -1026,6 +1050,22 @@ def serialize_nxm_match(rule, buf, offset):
         if header != 0:
             offset += nxm_put(buf, offset, header, rule)
 
+    if rule.flow.tcp_flags != 0:
+        # TCP Flags can only be used if the ethernet type is IPv4 or IPv6
+        if rule.flow.dl_type in (ether.ETH_TYPE_IP, ether.ETH_TYPE_IPV6):
+            # TCP Flags can only be used if the ip protocol is TCP
+            if rule.flow.nw_proto == inet.IPPROTO_TCP:
+                if rule.wc.tcp_flags_mask == UINT16_MAX:
+                    header = ofproto_v1_0.NXM_NX_TCP_FLAGS
+                else:
+                    header = ofproto_v1_0.NXM_NX_TCP_FLAGS_W
+            else:
+                header = 0
+        else:
+            header = 0
+        if header != 0:
+            offset += nxm_put(buf, offset, header, rule)
+
     # IP Source and Destination
     if rule.flow.nw_src != 0:
         if rule.wc.nw_src_mask == UINT32_MAX:
