@@ -1,3 +1,5 @@
+from __future__ import absolute_import
+
 import logging
 import pprint
 
@@ -9,14 +11,16 @@ from ryu.services.protocols.bgp.operator.commands.responses import \
     WrongParamResp
 from ryu.services.protocols.bgp.operator.views.conf import ConfDetailView
 from ryu.services.protocols.bgp.operator.views.conf import ConfDictView
-from route_formatter_mixin import RouteFormatterMixin
+from .route_formatter_mixin import RouteFormatterMixin
 
 LOG = logging.getLogger('bgpspeaker.operator.commands.show.vrf')
 
+SUPPORTED_VRF_RF = ('ipv4', 'ipv6', 'evpn')
+
 
 class Routes(Command, RouteFormatterMixin):
     help_msg = 'show routes present for vrf'
-    param_help_msg = '<vpn-name> <route-family>(ipv4, ipv6)'
+    param_help_msg = '<vpn-name> <route-family>%s' % str(SUPPORTED_VRF_RF)
     command = 'routes'
 
     def __init__(self, *args, **kwargs):
@@ -30,8 +34,9 @@ class Routes(Command, RouteFormatterMixin):
             return WrongParamResp()
         vrf_name = params[0]
         vrf_rf = params[1]
-        if vrf_rf not in ('ipv4', 'ipv6'):
-            return WrongParamResp('route-family not one of (ipv4, ipv6)')
+        if vrf_rf not in SUPPORTED_VRF_RF:
+            return WrongParamResp('route-family not one of %s' %
+                                  str(SUPPORTED_VRF_RF))
 
         from ryu.services.protocols.bgp.operator.internal_api import \
             WrongParamError
@@ -70,13 +75,15 @@ class Routes(Command, RouteFormatterMixin):
             if resp.status == STATUS_ERROR:
                 return Command.cli_resp_formatter(resp)
             ret = cls._format_family_header()
-            for family, data in resp.value.iteritems():
+            for family, data in resp.value.items():
                 ret += 'VPN: {0}\n'.format(family)
                 ret += cls._format_family(data)
             return ret
 
 
 class CountRoutesMixin(object):
+    api = None  # not assigned yet
+
     def _count_routes(self, vrf_name, vrf_rf):
         return len(self.api.get_single_vrf_routes(vrf_name, vrf_rf))
 
@@ -130,7 +137,7 @@ class Summary(Command, CountRoutesMixin):
             vrf_confs = self.api.get_vrfs_conf()
             view = ConfDictView(vrf_confs)
             encoded = view.encode()
-            for vrf_key, conf in encoded.iteritems():
+            for vrf_key, conf in encoded.items():
                 vrf_name, vrf_rf = vrf_key
                 conf['routes_count'] = self._count_routes(
                     vrf_name,
@@ -138,7 +145,7 @@ class Summary(Command, CountRoutesMixin):
                 )
 
             encoded = dict([(str(k), v)
-                            for k, v in encoded.iteritems()])
+                            for k, v in encoded.items()])
             return CommandsResponse(
                 STATUS_OK,
                 encoded
