@@ -46,6 +46,11 @@ CAP_MBGP_IPV6 = 'cap_mbgp_ipv6'
 CAP_MBGP_VPNV4 = 'cap_mbgp_vpnv4'
 CAP_MBGP_VPNV6 = 'cap_mbgp_vpnv6'
 CAP_MBGP_EVPN = 'cap_mbgp_evpn'
+CAP_MBGP_IPV4FS = 'cap_mbgp_ipv4fs'
+CAP_MBGP_IPV6FS = 'cap_mbgp_ipv6fs'
+CAP_MBGP_VPNV4FS = 'cap_mbgp_vpnv4fs'
+CAP_MBGP_VPNV6FS = 'cap_mbgp_vpnv6fs'
+CAP_MBGP_L2VPNFS = 'cap_mbgp_l2vpnfs'
 CAP_RTC = 'cap_rtc'
 RTC_AS = 'rtc_as'
 HOLD_TIME = 'hold_time'
@@ -562,7 +567,7 @@ def validate_conf_desc(description):
 
 @validate(name=ConfWithStats.STATS_LOG_ENABLED)
 def validate_stats_log_enabled(stats_log_enabled):
-    if stats_log_enabled not in (True, False):
+    if not isinstance(stats_log_enabled, bool):
         raise ConfigTypeError(desc='Statistics log enabled settings can only'
                               ' be boolean type.')
     return stats_log_enabled
@@ -582,7 +587,7 @@ def validate_stats_time(stats_time):
 
 @validate(name=CAP_REFRESH)
 def validate_cap_refresh(crefresh):
-    if crefresh not in (True, False):
+    if not isinstance(crefresh, bool):
         raise ConfigTypeError(desc='Invalid Refresh capability settings: %s. '
                               'Boolean value expected' % crefresh)
     return crefresh
@@ -590,7 +595,7 @@ def validate_cap_refresh(crefresh):
 
 @validate(name=CAP_ENHANCED_REFRESH)
 def validate_cap_enhanced_refresh(cer):
-    if cer not in (True, False):
+    if not isinstance(cer, bool):
         raise ConfigTypeError(desc='Invalid Enhanced Refresh capability '
                               'settings: %s. Boolean value expected' % cer)
     return cer
@@ -598,7 +603,7 @@ def validate_cap_enhanced_refresh(cer):
 
 @validate(name=CAP_FOUR_OCTET_AS_NUMBER)
 def validate_cap_four_octet_as_number(cfoan):
-    if cfoan not in (True, False):
+    if not isinstance(cfoan, bool):
         raise ConfigTypeError(desc='Invalid Four-Octet AS Number capability '
                               'settings: %s boolean value expected' % cfoan)
     return cfoan
@@ -606,7 +611,7 @@ def validate_cap_four_octet_as_number(cfoan):
 
 @validate(name=CAP_MBGP_IPV4)
 def validate_cap_mbgp_ipv4(cmv4):
-    if cmv4 not in (True, False):
+    if not isinstance(cmv4, bool):
         raise ConfigTypeError(desc='Invalid MP-BGP IPv4 capability '
                               'settings: %s. Boolean value expected' % cmv4)
 
@@ -615,7 +620,7 @@ def validate_cap_mbgp_ipv4(cmv4):
 
 @validate(name=CAP_MBGP_IPV6)
 def validate_cap_mbgp_ipv6(cmv6):
-    if cmv6 not in (True, False):
+    if not isinstance(cmv6, bool):
         raise ConfigTypeError(desc='Invalid MP-BGP IPv6 capability '
                               'settings: %s. Boolean value expected' % cmv6)
 
@@ -624,7 +629,7 @@ def validate_cap_mbgp_ipv6(cmv6):
 
 @validate(name=CAP_MBGP_VPNV4)
 def validate_cap_mbgp_vpnv4(cmv4):
-    if cmv4 not in (True, False):
+    if not isinstance(cmv4, bool):
         raise ConfigTypeError(desc='Invalid MP-BGP VPNv4 capability '
                               'settings: %s. Boolean value expected' % cmv4)
 
@@ -633,7 +638,7 @@ def validate_cap_mbgp_vpnv4(cmv4):
 
 @validate(name=CAP_MBGP_VPNV6)
 def validate_cap_mbgp_vpnv6(cmv6):
-    if cmv6 not in (True, False):
+    if not isinstance(cmv6, bool):
         raise ConfigTypeError(desc='Invalid MP-BGP VPNv6 capability '
                               'settings: %s. Boolean value expected' % cmv6)
 
@@ -642,15 +647,60 @@ def validate_cap_mbgp_vpnv6(cmv6):
 
 @validate(name=CAP_MBGP_EVPN)
 def validate_cap_mbgp_evpn(cmevpn):
-    if cmevpn not in (True, False):
+    if not isinstance(cmevpn, bool):
         raise ConfigTypeError(desc='Invalid Ethernet VPN capability '
                               'settings: %s. Boolean value expected' % cmevpn)
     return cmevpn
 
 
+@validate(name=CAP_MBGP_IPV4FS)
+def validate_cap_mbgp_ipv4fs(cmv4fs):
+    if not isinstance(cmv4fs, bool):
+        raise ConfigTypeError(desc='Invalid MP-BGP '
+                              'IPv4 Flow Specification capability '
+                              'settings: %s. Boolean value expected' % cmv4fs)
+    return cmv4fs
+
+
+@validate(name=CAP_MBGP_IPV6FS)
+def validate_cap_mbgp_ipv6fs(cmv6fs):
+    if not isinstance(cmv6fs, bool):
+        raise ConfigTypeError(desc='Invalid MP-BGP '
+                              'IPv6 Flow Specification capability '
+                              'settings: %s. Boolean value expected' % cmv6fs)
+    return cmv6fs
+
+
+@validate(name=CAP_MBGP_VPNV4FS)
+def validate_cap_mbgp_vpnv4fs(cmv4fs):
+    if not isinstance(cmv4fs, bool):
+        raise ConfigTypeError(desc='Invalid MP-BGP '
+                              'VPNv4 Flow Specification capability '
+                              'settings: %s. Boolean value expected' % cmv4fs)
+    return cmv4fs
+
+
+@validate(name=CAP_MBGP_VPNV6FS)
+def validate_cap_mbgp_vpnv66fs(cmv6fs):
+    if not isinstance(cmv6fs, bool):
+        raise ConfigTypeError(desc='Invalid MP-BGP '
+                              'VPNv6 Flow Specification capability '
+                              'settings: %s. Boolean value expected' % cmv6fs)
+    return cmv6fs
+
+
+@validate(name=CAP_MBGP_L2VPNFS)
+def validate_cap_mbgp_l2vpnfs(cml2fs):
+    if not isinstance(cml2fs, bool):
+        raise ConfigTypeError(desc='Invalid MP-BGP '
+                              'L2VPN Flow Specification capability '
+                              'settings: %s. Boolean value expected' % cml2fs)
+    return cml2fs
+
+
 @validate(name=CAP_RTC)
 def validate_cap_rtc(cap_rtc):
-    if cap_rtc not in (True, False):
+    if not isinstance(cap_rtc, bool):
         raise ConfigTypeError(desc='Invalid type for specifying RTC '
                               'capability. Expected boolean got: %s' %
                               type(cap_rtc))
