@@ -37,9 +37,17 @@ LOG = logging.getLogger('bgpspeaker.rtconf.common')
 # Global configuration settings.
 LOCAL_AS = 'local_as'
 ROUTER_ID = 'router_id'
+CLUSTER_ID = 'cluster_id'
 LABEL_RANGE = 'label_range'
 LABEL_RANGE_MAX = 'max'
 LABEL_RANGE_MIN = 'min'
+LOCAL_PREF = 'local_pref'
+
+# Similar to Cisco command 'allowas-in'. Allows the local ASN in the path.
+# Facilitates auto rd, auto rt import/export
+# ("rd auto/route-target both auto") and simplified spine/leaf architectures,
+# sharing an ASN between e.g. leafs.
+ALLOW_LOCAL_AS_IN_COUNT = 'allow_local_as_in_count'
 
 # Configuration that can be set at global level as well as per context
 # (session/vrf) level
@@ -78,6 +86,18 @@ DEFAULT_TCP_CONN_TIMEOUT = 30
 DEFAULT_BGP_CONN_RETRY_TIME = 30
 DEFAULT_MED = 0
 DEFAULT_MAX_PATH_EXT_RTFILTER_ALL = True
+DEFAULT_LOCAL_PREF = 100
+
+
+@validate(name=ALLOW_LOCAL_AS_IN_COUNT)
+def validate_allow_local_as_in_count(count):
+    if not isinstance(count, numbers.Integral):
+        raise ConfigTypeError(desc=('Configuration value for %s has to be '
+                                    'integral type' % ALLOW_LOCAL_AS_IN_COUNT))
+    if count < 0:
+        raise ConfigValueError(desc='Invalid local AS count %s' % count)
+
+    return count
 
 
 @validate(name=LOCAL_AS)
@@ -104,6 +124,16 @@ def validate_router_id(router_id):
     return router_id
 
 
+@validate(name=CLUSTER_ID)
+def validate_router_id(cluster_id):
+    if not isinstance(cluster_id, str):
+        raise ConfigTypeError(conf_name=CLUSTER_ID)
+    if not is_valid_ipv4(cluster_id):
+        raise ConfigValueError(desc='Invalid cluster id %s' % cluster_id)
+
+    return cluster_id
+
+
 @validate(name=REFRESH_STALEPATH_TIME)
 def validate_refresh_stalepath_time(rst):
     if not isinstance(rst, numbers.Integral):
@@ -184,13 +214,22 @@ def validate_bgp_conn_retry_time(bgp_conn_retry_time):
 
 @validate(name=MAX_PATH_EXT_RTFILTER_ALL)
 def validate_max_path_ext_rtfilter_all(max_path_ext_rtfilter_all):
-    if max_path_ext_rtfilter_all not in (True, False):
+    if not isinstance(max_path_ext_rtfilter_all, bool):
         raise ConfigTypeError(desc=('Invalid max_path_ext_rtfilter_all'
                                     ' configuration value %s' %
                                     max_path_ext_rtfilter_all))
     return max_path_ext_rtfilter_all
 
 
+@validate(name=LOCAL_PREF)
+def validate_local_pref(local_pref):
+    if not isinstance(local_pref, numbers.Integral):
+        raise ConfigTypeError(desc=('Invalid local_pref'
+                                    ' configuration value %s' %
+                                    local_pref))
+    return local_pref
+
+
 class CommonConf(BaseConf):
     """Encapsulates configurations applicable to all peer sessions.
 
@@ -208,13 +247,18 @@ class CommonConf(BaseConf):
                                    LABEL_RANGE, BGP_SERVER_PORT,
                                    TCP_CONN_TIMEOUT,
                                    BGP_CONN_RETRY_TIME,
-                                   MAX_PATH_EXT_RTFILTER_ALL])
+                                   MAX_PATH_EXT_RTFILTER_ALL,
+                                   ALLOW_LOCAL_AS_IN_COUNT,
+                                   CLUSTER_ID,
+                                   LOCAL_PREF])
 
     def __init__(self, **kwargs):
         super(CommonConf, self).__init__(**kwargs)
 
     def _init_opt_settings(self, **kwargs):
         super(CommonConf, self)._init_opt_settings(**kwargs)
+        self._settings[ALLOW_LOCAL_AS_IN_COUNT] = compute_optional_conf(
+            ALLOW_LOCAL_AS_IN_COUNT, 0, **kwargs)
         self._settings[LABEL_RANGE] = compute_optional_conf(
             LABEL_RANGE, DEFAULT_LABEL_RANGE, **kwargs)
         self._settings[REFRESH_STALEPATH_TIME] = compute_optional_conf(
@@ -230,6 +274,10 @@ class CommonConf(BaseConf):
         self._settings[MAX_PATH_EXT_RTFILTER_ALL] = compute_optional_conf(
             MAX_PATH_EXT_RTFILTER_ALL, DEFAULT_MAX_PATH_EXT_RTFILTER_ALL,
             **kwargs)
+        self._settings[CLUSTER_ID] = compute_optional_conf(
+            CLUSTER_ID, kwargs[ROUTER_ID], **kwargs)
+        self._settings[LOCAL_PREF] = compute_optional_conf(
+            LOCAL_PREF, DEFAULT_LOCAL_PREF, **kwargs)
 
     # =========================================================================
     # Required attributes
@@ -246,6 +294,13 @@ class CommonConf(BaseConf):
     # =========================================================================
     # Optional attributes with valid defaults.
     # =========================================================================
+    @property
+    def cluster_id(self):
+        return self._settings[CLUSTER_ID]
+
+    @property
+    def allow_local_as_in_count(self):
+        return self._settings[ALLOW_LOCAL_AS_IN_COUNT]
 
     @property
     def bgp_conn_retry_time(self):
@@ -275,6 +330,10 @@ class CommonConf(BaseConf):
     def max_path_ext_rtfilter_all(self):
         return self._settings[MAX_PATH_EXT_RTFILTER_ALL]
 
+    @property
+    def local_pref(self):
+        return self._settings[LOCAL_PREF]
+
     @classmethod
     def get_opt_settings(self):
         self_confs = super(CommonConf, self).get_opt_settings()
