@@ -138,9 +138,9 @@ class RouteTargetManager(object):
             self.add_rt_nlri(removed_rt, is_withdraw=True)
 
     def on_rt_filter_chg_sync_peer(self, peer, new_rts, old_rts, table):
-        LOG.debug('RT Filter changed for peer %s, new_rts %s, old_rts %s ' %
-                  (peer, new_rts, old_rts))
-        for dest in table.itervalues():
+        LOG.debug('RT Filter changed for peer %s, new_rts %s, old_rts %s ',
+                  peer, new_rts, old_rts)
+        for dest in table.values():
             # If this destination does not have best path, we ignore it
             if not dest.best_path:
                 continue
@@ -161,7 +161,7 @@ class RouteTargetManager(object):
                 if desired_rts.intersection(new_rts):
                     peer.communicate_path(dest.best_path)
 
-    def _compute_global_intrested_rts(self):
+    def _compute_global_interested_rts(self):
         """Computes current global interested RTs for global tables.
 
         Computes interested RTs based on current RT filters for peers. This
@@ -187,7 +187,7 @@ class RouteTargetManager(object):
         interesting RTs.
         """
         prev_global_rts = self._global_interested_rts
-        curr_global_rts = self._compute_global_intrested_rts()
+        curr_global_rts = self._compute_global_interested_rts()
 
         new_global_rts = curr_global_rts - prev_global_rts
         removed_global_rts = prev_global_rts - curr_global_rts
@@ -195,8 +195,8 @@ class RouteTargetManager(object):
         # Update current interested RTs for next iteration
         self._global_interested_rts = curr_global_rts
 
-        LOG.debug('Global Interested RT changed, new RTs %s, removed RTs %s' %
-                  (new_global_rts, removed_global_rts))
+        LOG.debug('Global Interested RT changed, new RTs %s, removed RTs %s',
+                  new_global_rts, removed_global_rts)
         tm = self._core_service.table_manager
         tm.on_interesting_rts_change(new_global_rts, removed_global_rts)
 
