@@ -25,7 +25,7 @@ LOG = logging.getLogger('test_ofproto_common')
 
 
 class TestOfprotCommon(unittest.TestCase):
-    """ Test case for ofprotp_common
+    """ Test case for ofproto_common
     """
 
     def test_struct_ofp_header(self):
@@ -33,5 +33,5 @@ class TestOfprotCommon(unittest.TestCase):
         eq_(OFP_HEADER_SIZE, 8)
 
     def test_define_constants(self):
-        eq_(OFP_TCP_PORT, 6633)
-        eq_(OFP_SSL_PORT, 6633)
+        eq_(OFP_TCP_PORT, 6653)
+        eq_(OFP_SSL_PORT, 6653)
