@@ -17,11 +17,11 @@
 
 import unittest
 import logging
+import six
 import struct
 import netaddr
 from struct import *
 from nose.tools import *
-from nose.plugins.skip import Skip, SkipTest
 from ryu.ofproto import ether, inet
 from ryu.lib.packet.ethernet import ethernet
 from ryu.lib.packet.packet import Packet
@@ -91,7 +91,7 @@ class Test_ethernet(unittest.TestCase):
     def test_default_args(self):
         e = ethernet()
         buf = e.serialize(bytearray(), None)
-        res = struct.unpack(e._PACK_STR, str(buf))
+        res = struct.unpack(e._PACK_STR, six.binary_type(buf))
 
         eq_(res[0], addrconv.mac.text_to_bin('ff:ff:ff:ff:ff:ff'))
         eq_(res[1], addrconv.mac.text_to_bin('00:00:00:00:00:00'))
